// MSRItfUSBHID.cpp: Implementation of the CMSRItfUSBHID class.
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "MSRItfUSBHID.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

extern int nReader;
//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CMSRItfUSBHID::CMSRItfUSBHID(INTERFACE_TYPE emItfType){
//	ASSERT(emItfType==ITF_TYPE_USB_HID_KB || emItfType==ITF_TYPE_USB_HID);
	if(emItfType != ITF_TYPE_USB_HID_KB && emItfType != ITF_TYPE_USB_HID)		emItfType = ITF_TYPE_USB_HID; // Default interface is ITF_TYPE_USB_HID.

	m_emItfType = emItfType; // Store interface type as class member.
	m_bPortOpened = FALSE; // Close port during initialization.

	m_hMSRDev = INVALID_HANDLE_VALUE; // Initialize device handle.
	m_strMSRName.Empty(); // Initialize device name string.

	m_pPreprstData = NULL;
	m_pHidCaps = NULL;
	m_VendorIDTmp = NULL;
	m_USBHID_PIDTmp = NULL;
	m_MAGTUSBHID_PIDTmp = NULL;
	m_USBHIDKB_PIDTmp = NULL;
	m_MOIR_flg = NULL;
}

CMSRItfUSBHID::~CMSRItfUSBHID(){
	if(m_hMSRDev != NULL)		CloseMSRDevice(); // Close device.
}

//////////////////////////////////////////////////////////////////////
// Return value: TRUE, continue in loop to enumerate other devices. 
BOOL CMSRItfUSBHID::EnumMSRDevice2(INTERFACE_TYPE *pemItfType, ULONG ulIndex, CString& strDevName){
 	BOOL								bRet = FALSE;
	DWORD								dwErr = 0;
	
	GUID								HidGuid;
	SP_INTERFACE_DEVICE_DATA			DevInfoData;
	HDEVINFO							hHwDevInfo = INVALID_HANDLE_VALUE ;
    PSP_INTERFACE_DEVICE_DETAIL_DATA    pDetailData = NULL;
    ULONG                               ulPredictedLength = 0;
    ULONG                               ulRequiredLength = 0;
	HANDLE								hMSR = INVALID_HANDLE_VALUE;
	HIDD_ATTRIBUTES						HidAttr;
	
	//Initialization
	if(pemItfType != NULL)
		*pemItfType = ITF_TYPE_UNKNOWN;
	strDevName.Empty();
	
	//Get Guid of HID class
	HidD_GetHidGuid(&HidGuid);
	
	// Open a handle to the plug and play dev node.
	hHwDevInfo = SetupDiGetClassDevs (
		&HidGuid,
		NULL, // Define no enumerator (global)
		NULL, // Define no
		(DIGCF_PRESENT | // Only Devices present
		DIGCF_INTERFACEDEVICE)); // Function class devices.
	
	if(hHwDevInfo == INVALID_HANDLE_VALUE)
	{
		dwErr = GetLastError();
		return FALSE;
	}
	
	DevInfoData.cbSize = sizeof (SP_INTERFACE_DEVICE_DATA);
	bRet = SetupDiEnumDeviceInterfaces (hHwDevInfo,
		0, // No care about specific PDOs
		&HidGuid,
		ulIndex,
		&DevInfoData);
	if(bRet == FALSE)
	{
		//bRet = FALSE;
		goto MY_EXIT;
	}
	
	//Continue to enumerate...
	bRet = TRUE;
	
	// allocate a function class device data structure to receive the
	// goods about this particular device.
	if( SetupDiGetInterfaceDeviceDetail (
		hHwDevInfo,
		&DevInfoData,
		NULL, // probing so no output buffer yet
		0, // probing so output buffer length of zero
		&ulRequiredLength,
		NULL ) == FALSE )				
	{
		dwErr = GetLastError();
		if(dwErr != ERROR_INSUFFICIENT_BUFFER)
			goto MY_EXIT;	
		
		ulPredictedLength = ulRequiredLength;
	} //end if
	
	
	pDetailData = (PSP_INTERFACE_DEVICE_DETAIL_DATA) malloc (ulPredictedLength);
	//	ASSERT(pDetailData != NULL);
	pDetailData->cbSize = sizeof (SP_INTERFACE_DEVICE_DETAIL_DATA);
	
	// Retrieve the information from Plug and Play.
	if( SetupDiGetInterfaceDeviceDetail (
		hHwDevInfo,
		&DevInfoData,
		pDetailData,
		ulPredictedLength,
		&ulRequiredLength,
		NULL	) == FALSE )
	{
		goto MY_EXIT;
	}
	
	
	//open the MSR
	hMSR = CreateFile(	pDetailData->DevicePath,
		0, //GENERIC_READ | GENERIC_WRITE,
		FILE_SHARE_READ | FILE_SHARE_WRITE,
		NULL, // no SECURITY_ATTRIBUTES structure
		OPEN_EXISTING, // No special create flags
		0, //FILE_FLAG_OVERLAPPED, // No special attributes
		NULL); // No template file
	if(hMSR == INVALID_HANDLE_VALUE)
		goto MY_EXIT;
	
	//Get Attributes
	memset(&HidAttr, 0, sizeof(HidAttr));
	if( HidD_GetAttributes(hMSR, &HidAttr) == FALSE )
		goto MY_EXIT;
	
	//if IDTECH and supported Product ID
	if((	HidAttr.VendorID == IDTECH_VENDOR_ID )
		|| (	HidAttr.VendorID == MOIR_VENDOR_ID )
		|| (	HidAttr.VendorID == HP_VENDOR_ID ))
	{
		m_VendorIDTmp = HidAttr.VendorID;
		if(	HidAttr.VendorID == IDTECH_VENDOR_ID ){
			if((HidAttr.ProductID ==IDTECH_USBHIDKB_PIDSRED)||(HidAttr.ProductID == IDTECH_USBHIDKB_PID) || (HidAttr.ProductID == IDTECH_USBHIDKB_PID2) || ( HidAttr.ProductID == IDTECH_USBHID_PID) || ( HidAttr.ProductID == IDTECH_USBHID_PID2) || (HidAttr.ProductID == MAGTEK_USBHID_PID)|| (HidAttr.ProductID == MAGTEK_USBHID_PID2)||(HidAttr.ProductID == IDTECH_USBHID_PIDSRED)){
				//			m_USBHID_PIDTmp = IDTECH_USBHID_PID;
				//			m_MAGTUSBHID_PIDTmp = MAGTEK_USBHID_PID;
				
				// SecureMag USB HID (MagTek)
				if (HidAttr.ProductID == MAGTEK_USBHID_PID)
				{
					m_MAGTUSBHID_PIDTmp = MAGTEK_USBHID_PID;
					
				}
				else if (HidAttr.ProductID ==IDTECH_USBHIDKB_PIDSRED)
				{
					m_USBHIDKB_PIDTmp = IDTECH_USBHIDKB_PIDSRED;
				}
				else if (HidAttr.ProductID ==IDTECH_USBHID_PIDSRED)
				{
					m_USBHID_PIDTmp = IDTECH_USBHID_PIDSRED;
					nReader = 1;
				}
				// ValueMag USB HID (MagTek)
				else if (HidAttr.ProductID == MAGTEK_USBHID_PID2)
				{
					m_MAGTUSBHID_PIDTmp = MAGTEK_USBHID_PID2;
					
				}
				
				// SecureMag USB HID KB
				else if (HidAttr.ProductID == IDTECH_USBHIDKB_PID)
				{
					m_USBHIDKB_PIDTmp = IDTECH_USBHIDKB_PID;
					nReader = 1;
				}
				
				// ValueMag USB HID KB
				else if (HidAttr.ProductID == IDTECH_USBHIDKB_PID2)
				{
					m_USBHIDKB_PIDTmp = IDTECH_USBHIDKB_PID2;
					nReader = 2;
				}
				
				// SecureMag USB HID 
				else if (HidAttr.ProductID == IDTECH_USBHID_PID)
				{
					m_USBHID_PIDTmp = IDTECH_USBHID_PID;
					nReader = 1;
				}
				
				// ValueMag USB HID
				else if (HidAttr.ProductID == IDTECH_USBHID_PID2)
				{
					m_USBHID_PIDTmp = IDTECH_USBHID_PID2;
					nReader = 2;
				}
				m_MOIR_flg = 0;
			}
			
		}
		/*
		else if(	HidAttr.VendorID == HP_VENDOR_ID ){
		m_USBHID_PIDTmp = HP_USBHID_PID;
		m_MAGTUSBHID_PIDTmp = HP_MAGTEK_USBHID_PID;
		m_USBHIDKB_PIDTmp = HP_USBHIDKB_PID;
		m_MOIR_flg = 0;
		}
		*/
		if (HidAttr.ProductID == m_USBHIDKB_PIDTmp)
		{
			strDevName = pDetailData->DevicePath;
			if(pemItfType != NULL)
				*pemItfType = ITF_TYPE_USB_HID_KB;
		}
		else if( HidAttr.ProductID == m_USBHID_PIDTmp || HidAttr.ProductID == m_MAGTUSBHID_PIDTmp) // Add m_MAGTUSBHID_PIDTmp
		{
			strDevName = pDetailData->DevicePath;
			if(pemItfType != NULL)
				*pemItfType = ITF_TYPE_USB_HID;
		}
	}
MY_EXIT:
	if(hMSR != INVALID_HANDLE_VALUE)
	{
		CloseHandle(hMSR);
		hMSR = INVALID_HANDLE_VALUE;
	}
	if(hHwDevInfo != INVALID_HANDLE_VALUE)
	{
		SetupDiDestroyDeviceInfoList(hHwDevInfo);
		hHwDevInfo = INVALID_HANDLE_VALUE;
	}
	
	if(pDetailData != NULL)
	{
		free(pDetailData);
		pDetailData = NULL;
	}	
	
	return bRet;
}


BOOL CMSRItfUSBHID::Index2Name(ULONG ulIndex, CString& strName)
{
 	BOOL	bRet = FALSE;
	DWORD	dwErr = 0;

 	GUID	HidGuid;
 	SP_INTERFACE_DEVICE_DATA			DevInfoData;
	HDEVINFO							hHwDevInfo = INVALID_HANDLE_VALUE ;
    PSP_INTERFACE_DEVICE_DETAIL_DATA    pDetailData = NULL;
    ULONG                               ulPredictedLength = 0;
    ULONG                               ulRequiredLength = 0;
  
	//Initialization
	strName.Empty();

	//Get Guid of HID class
	HidD_GetHidGuid(&HidGuid);

	// Open a handle to the plug and play dev node.
	hHwDevInfo = SetupDiGetClassDevs (
                           &HidGuid,
                           NULL, // Define no enumerator (global)
                           NULL, // Define no
                           (DIGCF_PRESENT | // Only Devices present
                           DIGCF_INTERFACEDEVICE)); // Function class devices.

	if(hHwDevInfo == INVALID_HANDLE_VALUE)
	{
		 dwErr = GetLastError();
		 return FALSE;
	}

	DevInfoData.cbSize = sizeof (SP_INTERFACE_DEVICE_DATA);
	bRet = SetupDiEnumDeviceInterfaces (hHwDevInfo,
                                         0, // No care about specific PDOs
                                         &HidGuid,
                                         ulIndex,
                                         &DevInfoData);
	if(bRet == FALSE)
	{
		//bRet = FALSE;
		goto MY_EXIT;
	}

	// allocate a function class device data structure to receive the
	// goods about this particular device.
	bRet = SetupDiGetInterfaceDeviceDetail (
				hHwDevInfo,
				&DevInfoData,
				NULL, // probing so no output buffer yet
				0, // probing so output buffer length of zero
				&ulRequiredLength,
				NULL); // not interested in the specific dev-node
	
	if(bRet == FALSE)
	{
		dwErr = GetLastError();
		if(dwErr != ERROR_INSUFFICIENT_BUFFER)
		{
			//bRet = FALSE;
			goto MY_EXIT;	
		}

		ulPredictedLength = ulRequiredLength;
	} //end if


	pDetailData = (PSP_INTERFACE_DEVICE_DETAIL_DATA) malloc (ulPredictedLength);
	pDetailData->cbSize = sizeof (SP_INTERFACE_DEVICE_DETAIL_DATA);

	// Retrieve the information from Plug and Play.
	bRet = SetupDiGetInterfaceDeviceDetail (
			   hHwDevInfo,
			   &DevInfoData,
			   pDetailData,
			   ulPredictedLength,
			   &ulRequiredLength,
			   NULL);

	if(bRet == TRUE)
		strName = pDetailData->DevicePath;

MY_EXIT:

	if(hHwDevInfo != INVALID_HANDLE_VALUE)
	{
		SetupDiDestroyDeviceInfoList(hHwDevInfo);
		hHwDevInfo = INVALID_HANDLE_VALUE;
	}

	if(pDetailData != NULL)
	{
		free(pDetailData);
 		pDetailData = NULL;
	}
	
	return bRet;
}


BOOL CMSRItfUSBHID::OpenMSRDevice( ULONG ulIndex )
{
	BOOL bRet = FALSE;
	CString strDevName;

	if(m_bPortOpened)
		return TRUE;

	m_ulIndex = ulIndex;

	//Index => DeviceName
	bRet = Index2Name(ulIndex, strDevName);
	if(bRet == FALSE || strDevName.IsEmpty())
		return FALSE;

	m_bPortOpened = OpenMSRDeviceByName(strDevName);
	return m_bPortOpened;
}

BOOL CMSRItfUSBHID::OpenMSRDeviceByName( LPCTSTR lpszName)
{
	BOOL bRet = FALSE;

	USHORT usPID=0, usPID2=0; //for supporting m_MAGTUSBHID_PIDTmp
	CString strDevName;
	DWORD dwAccess=0;
	HANDLE hMSR = INVALID_HANDLE_VALUE;
	HIDD_ATTRIBUTES HidAttr;
	NTSTATUS nStatus;
	
	if(lpszName==NULL || lstrlen(lpszName)==0)
		return FALSE;

	strDevName = lpszName;
	if(m_strMSRName.CompareNoCase(strDevName) == 0)
	{
//		ASSERT(m_hMSRDev != INVALID_HANDLE_VALUE);
		return TRUE;
	}
	else
	{
		if(m_hMSRDev != INVALID_HANDLE_VALUE)
			CloseMSRDevice();
	}

	//Interface Type and its desired access
	switch(m_emItfType) //Set outside!!!
	{
	case ITF_TYPE_USB_HID_KB:
		usPID = m_USBHIDKB_PIDTmp;
		dwAccess = 0;
		break;
	case ITF_TYPE_USB_HID:
		usPID = m_USBHID_PIDTmp;
		usPID2 = m_MAGTUSBHID_PIDTmp;
		dwAccess = GENERIC_READ | GENERIC_WRITE;
		break;
	}


	//open the MSR
	hMSR = CreateFile(	strDevName,
						dwAccess, //GENERIC_READ | GENERIC_WRITE,
						FILE_SHARE_READ | FILE_SHARE_WRITE,
						NULL, // no SECURITY_ATTRIBUTES structure
						OPEN_EXISTING, // No special create flags
						0, //FILE_FLAG_OVERLAPPED, // No special attributes
						NULL); // No template file
	if(hMSR == INVALID_HANDLE_VALUE)
		return FALSE;

	//Get Attributes
	memset(&HidAttr, 0, sizeof(HidAttr));
	bRet = HidD_GetAttributes(hMSR, &HidAttr);
	if(bRet == FALSE)
	{	
		//bRet = FALSE;
		goto MY_EXIT;
	}

	//if IDTECH and supported Product ID
	if((	HidAttr.VendorID != IDTECH_VENDOR_ID ) || (HidAttr.ProductID != usPID	&& HidAttr.ProductID != usPID2) )
	{
		bRet = FALSE;
		goto MY_EXIT;
	}

//	ASSERT(m_pPreprstData == NULL);
	bRet = HidD_GetPreparsedData(hMSR, &m_pPreprstData);
	if(bRet==FALSE || m_pPreprstData==NULL)
	{
		bRet = FALSE;
		goto MY_EXIT;
	}
	
//	ASSERT(m_pHidCaps == NULL);
	m_pHidCaps = (PHIDP_CAPS)calloc(1, sizeof(HIDP_CAPS));
	if(m_pHidCaps == NULL)
	{
		bRet = FALSE;
		goto MY_EXIT;
	}
	nStatus	= HidP_GetCaps(m_pPreprstData, m_pHidCaps);
	if(nStatus != HIDP_STATUS_SUCCESS)
	{
		bRet = FALSE;
		goto MY_EXIT;
	}
	
	//set related member data
	m_strMSRName = strDevName;
	m_hMSRDev = hMSR;

	//DON'T FORGET!!
	bRet = TRUE;
	
MY_EXIT:
	if(bRet == FALSE)
	{
		if(m_pHidCaps != NULL)
		{
			free(m_pHidCaps);
			m_pHidCaps = NULL;
		}

		if(m_pPreprstData != NULL)
		{
			HidD_FreePreparsedData(m_pPreprstData);
			m_pPreprstData = NULL;
		}

		if(hMSR != INVALID_HANDLE_VALUE)
		{
			CloseHandle(hMSR);
			hMSR = INVALID_HANDLE_VALUE;
		}
	}

	return bRet;
}

BOOL CMSRItfUSBHID::SendCommand(BYTE* pCmdBuf,ULONG cbCmdBuf,PBYTE pRetBuf,PULONG pcbRetBuf)
{
#ifdef _DEBUG
#endif
	ULONG ulRptLen = 0;
	ULONG ulRptCnt = 0, i, j, k, counter = 0;
	PBYTE p=NULL;
	ULONG ulRetLen=0;
   	BOOLEAN	Result;
	BOOL bNAKReport=FALSE;
	unsigned char pRptBuf[9];
	if(pcbRetBuf == NULL)
		return FALSE;
	if(m_pHidCaps == NULL || m_hMSRDev == INVALID_HANDLE_VALUE)
	{
		*pcbRetBuf = 0;
		return FALSE;
	}
	
	ulRptLen = m_pHidCaps->FeatureReportByteLength; 
	if(pRptBuf == NULL)
	{
		*pcbRetBuf = 0;
		return FALSE;
	}
	
	//remove one byte occupied by ReportID
	ulRptLen--;
	ulRptCnt = (cbCmdBuf + ulRptLen - 1) / ulRptLen;
	p = pCmdBuf;
	
	//Send all the feature report
	for(i=0; i<ulRptCnt; i++)
	{
		p = pCmdBuf + i * ulRptLen;
		
		memset(pRptBuf, 0, ulRptLen+1);
		memcpy(	&pRptBuf[1], 
			p, 
			i==ulRptCnt-1 ? cbCmdBuf-i*ulRptLen : ulRptLen);
		
		//try 3 times
		for(j=0; j<3; j++)
		{
			Result = HidD_SetFeature(m_hMSRDev, pRptBuf, ulRptLen+1);
			if(Result)	break;
			Sleep(10); //10ms
		}
		if(j == 3)
		{
			Result = FALSE;
			
		}
		
	} //end for
	

	CString Strtemp;
	BOOL bDataReceived = FALSE;
	
	for(;;)
	{
	

		//try 3 times
		//memset(pRptBuf, 0, ulRptLen+1);
		const int COUNT = 200;
		for(j=0; j<COUNT; j++)
		{
		
			memset(pRptBuf, 0, ulRptLen+1);
			Result = HidD_GetFeature(m_hMSRDev,
				pRptBuf,
				ulRptLen+1);
		

			if(Result)
			{
			
				//whether all the successive char are 0 or not,
				//if NO, it is still valid response data...
				if ( pRptBuf[0] == 0 && pRptBuf[1] == 0 && pRptBuf[2] == 0 && pRptBuf[3] == 0 && 
					pRptBuf[4] == 0 && pRptBuf[5] == 0 && pRptBuf[6] == 0 && pRptBuf[7] == 0 )
				{
					if ( bDataReceived ) 
						goto SC_OUT;
					 Sleep(10);
					continue;
				}else
					break;
			}
		
		
		
		}
		
		if(ulRetLen + ulRptLen > 1023)
		{		
			TRACE("slop over!\n");
			Result = FALSE;
			OutputDebugString("7");
			break;
		}

		if(j == COUNT)
		{
			OutputDebugString("8");
			TRACE("20 times\n");
			break;
		}
		
		
		TRACE("to add this data\n");

		memcpy(&pRetBuf[ulRetLen], &pRptBuf[1], ulRptLen);
		ulRetLen += ulRptLen;
	
		bDataReceived = TRUE;
	}

SC_OUT:
	*pcbRetBuf = ulRetLen;
	
	
	return Result;
}

BOOL CMSRItfUSBHID::CloseMSRDevice()
{
	if(m_pHidCaps != NULL)
	{
		free(m_pHidCaps);
		m_pHidCaps = NULL;
	}

	if(m_pPreprstData != NULL)
	{
		HidD_FreePreparsedData(m_pPreprstData);
		m_pPreprstData = NULL;
	}

	if(m_hMSRDev != INVALID_HANDLE_VALUE)
	{
		CloseHandle(m_hMSRDev);
		m_hMSRDev = INVALID_HANDLE_VALUE;
	}

	m_strMSRName.Empty();
	m_bPortOpened = FALSE;

	return TRUE;
}

BOOL CMSRItfUSBHID::Send(BYTE* pCmdBuf,ULONG cbCmdBuf)
{
#ifdef _DEBUG
//TRACE("\nSendCommand:");
for(ULONG n=0;n<cbCmdBuf;n++)
{
//	if(!(n%16))
//		TRACE("\n");
//	TRACE("%02x ",pCmdBuf[n]);
}
//TRACE("\n");
#endif
	BOOL bRet=FALSE, bNAKReport=FALSE;
	ULONG ulRptLen = 0;
	ULONG ulRptCnt = 0, i, j;
	PBYTE pRptBuf=NULL, p=NULL;
	ULONG ulRetLen=0;

	if(m_pHidCaps == NULL || m_hMSRDev == INVALID_HANDLE_VALUE)
	{
		return FALSE;
	}

	ulRptLen = m_pHidCaps->FeatureReportByteLength; //??
	pRptBuf = (PBYTE)calloc(ulRptLen, sizeof(BYTE));
	if(pRptBuf == NULL)
	{
		return FALSE;
	}

	//remove one byte occupied by ReportID
	ulRptLen--;
	ulRptCnt = (cbCmdBuf + ulRptLen - 1) / ulRptLen;
	p = pCmdBuf;

	//Send all the feature report
	for(i=0; i<ulRptCnt; i++)
	{
		p = pCmdBuf + i * ulRptLen;

		memset(pRptBuf, 0, ulRptLen+1);
		//pRptBuf[0] = BYTE(i); //ReportID: 0 or i??
		memcpy(	&pRptBuf[1], 
				p, 
				i==ulRptCnt-1 ? cbCmdBuf-i*ulRptLen : ulRptLen);

		//try 3 times
		for(j=0; j<3; j++)
		{
			bRet = HidD_SetFeature(m_hMSRDev, pRptBuf, ulRptLen+1);
			if(bRet)	break;
			Sleep(10); //10ms
		}
		if(j == 3)
		{
			bRet = FALSE;			
		}

	} //end for
		
	CloseMSRDevice();
	return bRet;
}

                          
BOOL CMSRItfUSBHID::SendCommand2(BYTE* pCmdBuf,ULONG cbCmdBuf, CString &strResponse)
{



	strResponse.Empty();
	
	BYTE MyCmdBuf[1024], MyRetBuf[1024];
	ULONG ulMyCmdLen=1024, ulMyRetLen=1024;
	//	BYTE LRC = 0;
	
	strResponse.Empty();
	CString StringTmp;
	int LastByte;
	
	if(m_hMSRDev == INVALID_HANDLE_VALUE)
	{
		if(!OpenMSRDevice(m_ulIndex))
			return FALSE;
	}
	
	
	for(int i=0; i<(int)cbCmdBuf; i++)
		MyCmdBuf[i] = pCmdBuf[i];
	
	ulMyCmdLen = cbCmdBuf;
	
	
	//	ASSERT(bRet && ulMyCmdLen!=0);
	if(!SendCommand(MyCmdBuf, ulMyCmdLen, MyRetBuf, &ulMyRetLen))
		return FALSE;
	
	CloseMSRDevice();
	
	LastByte = ulMyRetLen;
	
	if(ulMyRetLen > (ULONG)0)
	{
		strResponse = _T("");
		for (i=ulMyRetLen-1; i>0; i--)		
			if (MyRetBuf[i] == 0)
				LastByte = i;
			else
				break;
	}

	if(MyRetBuf[0]==0x06&&MyRetBuf[1]!=0x02)
	{
      StringTmp.Format("%02X ",MyRetBuf[0]);
		strResponse += StringTmp;
		
	}
	else
	{
		int lrc=0;
	for (i=0; i<LastByte; i++)
	{
		if(MyRetBuf[i]==0x03)
		{
	
		
		for(int k=1;k<=i;k++)
        lrc=lrc^MyRetBuf[k];
		if(lrc==MyRetBuf[i+1])
		{
			StringTmp.Format("%02X %02X ",MyRetBuf[i],MyRetBuf[i+1]);
		strResponse += StringTmp;
			break;
		}
		else {
			
			StringTmp.Format("%02X ",MyRetBuf[i]);
			strResponse += StringTmp;
		}
		
		
		}
		else {
			StringTmp.Format("%02X ",MyRetBuf[i]);
			strResponse += StringTmp;}
		
	}
	}

	
	return !m_FirmwareVersion.IsEmpty();
	
}


BOOL CMSRItfUSBHID::SendCommand3(BYTE* pCmdBuf,ULONG cbCmdBuf,PBYTE pRetBuf,PULONG pcbRetBuf)
{
#ifdef _DEBUG
//TRACE("\nSendCommand:");
for(ULONG n=0;n<cbCmdBuf;n++)
{
//	if(!(n%16))
//		TRACE("\n");
//	TRACE("%02x ",pCmdBuf[n]);
}
//TRACE("\n");
#endif
	BOOL bRet=FALSE, bNAKReport=FALSE, bFirstNonZeroBlock = FALSE;
	ULONG ulRptLen = 0;
	ULONG ulRptCnt = 0, i, j, counter = 0;
	PBYTE pRptBuf=NULL, p=NULL;
	ULONG ulRetLen=0;

	if(pcbRetBuf == NULL)
		return FALSE;

	if(m_pHidCaps == NULL || m_hMSRDev == INVALID_HANDLE_VALUE)
	{
		*pcbRetBuf = 0;
		return FALSE;
	}

	ulRptLen = m_pHidCaps->FeatureReportByteLength; 
	pRptBuf = (PBYTE)calloc(ulRptLen, sizeof(BYTE));
	if(pRptBuf == NULL)
	{
		*pcbRetBuf = 0;
		return FALSE;
	}

	//remove one byte occupied by ReportID
	ulRptLen--;
	ulRptCnt = (cbCmdBuf + ulRptLen - 1) / ulRptLen;
	p = pCmdBuf;

	//Send all the feature report
	for(i=0; i<ulRptCnt; i++)
	{
		p = pCmdBuf + i * ulRptLen;

		memset(pRptBuf, 0, ulRptLen+1);
		memcpy(	&pRptBuf[1], 
				p, 
				i==ulRptCnt-1 ? cbCmdBuf-i*ulRptLen : ulRptLen);

		//try 3 times
		for(j=0; j<3; j++)
		{
			bRet = HidD_SetFeature(m_hMSRDev, pRptBuf, ulRptLen+1);
			if(bRet)	
				break;
			Sleep(10); //10ms
		}
		if(j == 3)
		{
			bRet = FALSE;
			goto MY_EXIT;
		}

	} //end for

	Sleep(100); // (1500); //JW

	//get response data from MSR
	int iEmptyCnt;
	bool bIsEmpty;
	iEmptyCnt = 0;
	while (1) {
//	for(k=0;k<500;k++) // 5 second in total
//	{

		bNAKReport = false;
		memset(pRptBuf, 0, ulRptLen+1);
		bRet = HidD_GetFeature(m_hMSRDev, pRptBuf, ulRptLen+1);
		bIsEmpty = true;
		if(bRet) {
					//whether all the successive char are 0 or not,
					//if NO, it is still valid response data...
			for(i=0; i<ulRptLen; i++) {
				if(pRptBuf[i] != 0)	{
					bFirstNonZeroBlock = true;
					bIsEmpty = false;
					iEmptyCnt = 0;
					break;
				}
			}
		}
				
//				if(i == ulRptLen) //  all 0, it is valid response data											
//					if (bFirstNonZeroBlock) {
//						counter++;
//						sleep(100);	// JW
//					}
		if(ulRetLen + ulRptLen > *pcbRetBuf)
		{			
			bRet = FALSE;
			goto MY_EXIT;
		}
		if(bIsEmpty) {
			Sleep(100);
			iEmptyCnt++;
			if((iEmptyCnt >= 35) ||						// 3.5 S if no response
				(iEmptyCnt >= 5 && bFirstNonZeroBlock)) // 0.5 S after response
				goto MY_EXIT;
		} 
		if (bFirstNonZeroBlock && bRet) {
			memcpy(&pRetBuf[ulRetLen], &pRptBuf[1], ulRptLen);
			ulRetLen += ulRptLen;
		} 
//		Sleep(10);
	} //while()

MY_EXIT:

	*pcbRetBuf = ulRetLen;

	if(pRptBuf != NULL)
	{
		free(pRptBuf);
		pRptBuf = NULL;
	}

	return bRet;
}