
#include "stdafx.h"
#include <stdio.h>
#include <malloc.h>

#include "USB.h"
#include <setupapi.h>

static const GUID GUID_DEVINTERFACE_USBPRINT = {0x28d78fad, 0x5a12, 0x11D1, {0xae, 0x5b, 0x00, 0x00, 0xf8, 0x03, 0xa8, 0xc2}};

HANDLE  gw_event;
HANDLE  gr_event;
OVERLAPPED g_olr;
OVERLAPPED g_olw;

//---------------------------------------------------------------------

CUsb::CUsb (){
	h = INVALID_HANDLE_VALUE;
	gw_event = CreateEvent(NULL,TRUE,FALSE,NULL);
	gr_event = CreateEvent(NULL,TRUE,FALSE,NULL);
} // CUsb::CUsb 
//---------------------------------------------------------------------

CUsb::~CUsb (){
	Close();
	if(gw_event != NULL)	CloseHandle(gw_event);
	if(gr_event != NULL)	CloseHandle(gr_event);
} // CUsb::~CUsb
//---------------------------------------------------------------------

Close();

BOOL	CUsb::Open()
{
  HDEVINFO devs;
  DWORD devcount;
  SP_DEVINFO_DATA devinfo;
  SP_DEVICE_INTERFACE_DATA devinterface;
  DWORD size;
  GUID intfce;
  PSP_DEVICE_INTERFACE_DETAIL_DATA interface_detail;

  intfce = GUID_DEVINTERFACE_USBPRINT;
  devs = SetupDiGetClassDevs(&intfce, 0, 0, DIGCF_PRESENT | DIGCF_DEVICEINTERFACE);
  if (devs == INVALID_HANDLE_VALUE) {
    return FALSE;
  }
  devcount = 0;
  devinterface.cbSize = sizeof(SP_DEVICE_INTERFACE_DATA);
  while (SetupDiEnumDeviceInterfaces(devs, 0, &intfce, devcount, &devinterface)) {
     // The following buffers would normally be malloced to he correct size
     // but here we just declare them as large stack variables
     // to make the code more readable
     //
    char driverkey[256];
    char interfacename[256];
    char location[256];

    // If this is not the device we want, we would normally continue onto the next one
    // so something like if (!required_device) continue; would be added here
    //
    devcount++;
    size = 0;
    // See how large a buffer we require for the device interface details
    SetupDiGetDeviceInterfaceDetail(devs, &devinterface, 0, 0, &size, 0);
    devinfo.cbSize = sizeof(SP_DEVINFO_DATA);
    interface_detail = (PSP_DEVICE_INTERFACE_DETAIL_DATA) calloc(1, size);
    if (interface_detail) {
      interface_detail->cbSize = sizeof (SP_DEVICE_INTERFACE_DETAIL_DATA);
      devinfo.cbSize = sizeof(SP_DEVINFO_DATA);
      if (!SetupDiGetDeviceInterfaceDetail(devs, &devinterface, interface_detail, size, &size, &devinfo)) {
	free(interface_detail);
	SetupDiDestroyDeviceInfoList(devs);
	return FALSE;
      }
      // Make a copy of the device path for later use
      strcpy(interfacename, interface_detail->DevicePath);
      free(interface_detail);
	  //compare it with the string VID&PID
	  CharLower(interfacename);	
	  if( _tcsstr(interfacename, _T("vid_0acd&pid_0930")) == NULL) // if NOT NULL, it is an IDTECH virtual printer device!
	  {
		SetupDiDestroyDeviceInfoList(devs);
		return FALSE;
	  }

      // And now fetch some useful registry entries
      size = sizeof(driverkey);
      driverkey[0] = 0;
      if (!SetupDiGetDeviceRegistryProperty(devs, &devinfo, SPDRP_DRIVER, 0, (LPBYTE)driverkey, size, 0)) {
	SetupDiDestroyDeviceInfoList(devs);
	return FALSE;
      }
      size = sizeof(location);
      location[0] = 0;
      if (!SetupDiGetDeviceRegistryProperty(devs, &devinfo, SPDRP_LOCATION_INFORMATION, 0, (LPBYTE)location, size, 0)) {
	SetupDiDestroyDeviceInfoList(devs);
	return FALSE;
      }
		if ( h != INVALID_HANDLE_VALUE )
		Close();

		h = CreateFile(interfacename, 
					GENERIC_WRITE | GENERIC_READ, 
					FILE_SHARE_WRITE | FILE_SHARE_READ,
			     (LPSECURITY_ATTRIBUTES)NULL,
				 OPEN_ALWAYS,
				 FILE_ATTRIBUTE_NORMAL | FILE_FLAG_SEQUENTIAL_SCAN | FILE_FLAG_OVERLAPPED, // FILE_FLAG_OVERLAPPED, 
				 NULL);
		if ( h == INVALID_HANDLE_VALUE )
		{
			return FALSE;
		}
		else
		{
		// Now perform all the writing to the device ie.
		// while (some condition) WriteFile(h, buf, size, &bytes_written);
			return TRUE;
		}
    }
  }
  SetupDiDestroyDeviceInfoList(devs);
  return FALSE;
} // CUsb::Open
//---------------------------------------------------------------------

BOOL	CUsb::Close(){
	if ( h == INVALID_HANDLE_VALUE ){
		return FALSE;
	}else{
		CloseHandle(h);
		h = INVALID_HANDLE_VALUE;
		return TRUE;
	}
} // CUsb::Close
//---------------------------------------------------------------------
unsigned long CUsb::Write(const void * data, unsigned int size)
{
	return (unsigned long)Send ((const char *)data, (DWORD) size );
}

BOOL	CUsb::Send (const char *pcInData, DWORD dwDataLen ) // Output data and output length
{
	if (h == INVALID_HANDLE_VALUE)
	{
		return FALSE;
	}

	if( !pcInData )
	{
		return FALSE;
	}

	if( dwDataLen == 0 )
		dwDataLen = strlen(pcInData);	//-- length can be ommited for null-terminated strings

	if( dwDataLen == 0 )
	{
		return FALSE;
	}

	DWORD	dwBytes = 0;
	memset ( &g_olw, 0, sizeof(g_olw) );
	g_olw.hEvent=gw_event;

	if(ResetEvent(g_olw.hEvent) == 0)
	{
		return FALSE;
	}

	DWORD dwComplete = WriteFile((HANDLE)h, pcInData, dwDataLen, &dwBytes, (LPOVERLAPPED) &g_olw);
	if(0 == dwComplete){	
        // Error, should wait until write operation is complete
		DWORD dwError=GetLastError();
		if (dwError != ERROR_IO_PENDING) {
			return FALSE;
		}

		DWORD dwResult = WaitForSingleObject(g_olw.hEvent, 5000);
		if(dwResult != WAIT_OBJECT_0){
			return FALSE;
		}

		if (!GetOverlappedResult(h, &g_olw, &dwBytes, FALSE)) {
			return FALSE;
					// Get out of loop and check for bytes written
		}
	}


	if( dwBytes == dwDataLen )
		return TRUE;
	else
	{
		return FALSE;
	}

} // CUsb::Send

unsigned long CUsb::Read(BYTE * data, unsigned int size){ // Input buffer and maximum length
	DWORD DataLen = 0;

	BOOL	bRet = TRUE;
	DWORD	dwBytes = 0;

	if (h == INVALID_HANDLE_VALUE)
	{
		return FALSE;
	}

	memset ( &g_olr, 0, sizeof(g_olr) );
	g_olr.hEvent=gr_event;

	DWORD dwComplete = ReadFile(h,	data,	size,	&dwBytes,	(LPOVERLAPPED) &g_olr);	
	if(0 == dwComplete){	
        // Error, should wait until write operation is complete
		DWORD dwError=GetLastError();
		if (dwError != ERROR_IO_PENDING) {
			return FALSE;
		}
//		return FALSE;
	}


// Wait the completion of calls and handle it
	DWORD dwResult = WaitForSingleObject(gr_event, 5000);
	switch(dwResult){
		case WAIT_TIMEOUT:
		return dwBytes;

		case WAIT_OBJECT_0: 
		if (!GetOverlappedResult(h, &g_olr, &dwBytes, FALSE)){
			return FALSE;
		}
		return dwBytes;

		default:
		return FALSE;
	}

	return FALSE;
} // CUsb::Receive 

//---------------------------------------------------------------------

unsigned long	CUsb::SendReceive (const char *pcInData, DWORD dwDataLen, BYTE * sOutData, BYTE WResp)
{	//-- this member works with binary data
	if( !pcInData || dwDataLen == 0 )
	{
		return 0;
	}

	unsigned long	bRet = 0;
		//-- Read any (unrelated) data that might be in the device's buffer
		//-- Send request and receive response
	Send( pcInData, dwDataLen );
	bRet = Read( sOutData, 4096);
	return bRet;

} // CUsb::SendReceive
//---------------------------------------------------------------------
