//************************************************************************************
//      Copyright (c) 2005 International Technologies & Systems corporation.
//      All rights reserved.
//
//      ID TECH is a registered trademark of International Technologies & Systems
//      Corporation.
//
//      This file is the confidential and proprietary property of ID TECH, Inc and the
//      posession or use of this file requires written license from ID TECH, Inc.
//
//************************************************************************************
//
//      Name:         dukpt.h
//      Last Update:  June 27, 2005
//      Description:  This is the public header file which exports interface calls to
//                    the DUKPT security module for PIN pad key management.
//      Status:       Under development
//
//************************************************************************************
//			
//	History:
//      Modified by Simon Lee - ID TECH
//	
//************************************************************************************
//

#ifndef _DUKPT_H_
#define _DUKPT_H_

//#include "define.h"
//#include "PINpad.h"
#define     INT8U                   unsigned char
#define     INT32S                  int
#define     INT32U                  unsigned int
/* return codes from export routine */
#define     DUKPT_ERR_UNDEFINED      0       /* Undefined error such as malloc failure, etc */
#define     DUKPT_ERR_BADPIN        -1      /* PIN number is invalid */
#define     DUKPT_ERR_BADACCT       -2      /* Account number is invalid */
#define     DUKPT_ERR_CHECKSUMACCT  -5      /* Account number fails check sum */
#define     DUKPT_ERR_BADKEY        -4      /* Invalid Dukpt Key */
#define     DUKPT_ERR_BADKSN        -6      /* Invalid Ksn Data */
#define     DUKPT_ERR_UNINITIALIZED -7      /* Initial Key not loaded */
#define     DUKPT_SUCCESS            1       /* No Error */

//#define DBG     // allows printing of debug message to output window or file.

// WYT Notes:
// The following flag enables swapping of the upper and lower 8 bytes of
// double length key using some convoluted hack. However, by applying the swapping,
// our key generation routine will generate the correct key sequence example
// provided by the X9 spec.
// Note: the swapping is only applicable in the 3DES case.
#define SWAP

#define SIZE_KSN_REG        10      // size of key serial number register
#define SIZE_KSN_MIN	    10		// min size
#define SIZE_KSN_MAX	    20		// max size
#define SIZE_ENC_CNTR       3       // size of encryption counter
#define SIZE_CRYP_REG       8       // size of crypto register (1 or 2)
#define SIZE_PIN_BLK        16      // pin block size
#define SIZE_3DES_KEY       16      // 3DES key size
#define SIZE_DES_KEY        8       // DES key size
#define NUM_FKEY_REG        21		// # of future key registers
#define NUM_BITS_EC         21      // # of valid bits in encryption counter

// The PINpad reaches end of life when the 21-bit encryption counter overruns
// i.e. when EC > 0x200000
#define MAX_EC              0x200000

// Bit position in shift register
// Note that the bit position are numbered from left to right as #1 to #21, with bit #1
// at bit 20 and #21 at bit 0.
#define BIT_1               (1L << 20)
#define BIT_21              (1L <<  0)

// Index to future key register
#define REG_1               0
#define REG_21              20

typedef  struct
{
    unsigned char   keytype;
    unsigned long   encrypt_ctr;
    unsigned char   future_key_reg[NUM_FKEY_REG][SIZE_3DES_KEY + 1];
    unsigned char   initKSN[SIZE_KSN_REG];
} dukpt_type;

//#define DBG     // allows printing of debug message to output window or file.

// WYT Notes:
// The following flag enables swapping of the upper and lower 8 bytes of
// double length key using some convoluted hack. However, by applying the swapping,
// our key generation routine will generate the correct key sequence example
// provided by the X9 spec.
// Note: the swapping is only applicable in the 3DES case.
#define SWAP

#define SIZE_KSN_REG        10      // size of key serial number register
#define SIZE_KSN_MIN	    10		// min size
#define SIZE_KSN_MAX	    20		// max size
#define SIZE_ENC_CNTR       3       // size of encryption counter
#define SIZE_CRYP_REG       8       // size of crypto register (1 or 2)
#define SIZE_PIN_BLK        16      // pin block size
#define SIZE_3DES_KEY       16      // 3DES key size
#define SIZE_DES_KEY        8       // DES key size
#define NUM_FKEY_REG        21		// # of future key registers
#define NUM_BITS_EC         21      // # of valid bits in encryption counter

// The PINpad reaches end of life when the 21-bit encryption counter overruns
// i.e. when EC > 0x200000
#define MAX_EC              0x200000

// Bit position in shift register
// Note that the bit position are numbered from left to right as #1 to #21, with bit #1
// at bit 20 and #21 at bit 0.
#define BIT_1               (1L << 20)
#define BIT_21              (1L <<  0)

// Index to future key register
#define REG_1               0
#define REG_21              20

// export routines
//#ifdef __cplusplus
//extern "C" {
//#endif
/*
//-----------------------------------------------------------------------------
// IDT_LoadInitKey:
// Initialize key serial number register with initial KSN value and 0 initial
// encryption counter value. Populate future key registers using the initial
// key.
// Note: values are passed to IDT_xxx routines in binary, with each byte
// containing 2 hex digit.
//-----------------------------------------------------------------------------
int IDT_LoadInitKey(
    int         isTDEA,         // indicate whether the key is single-
                                // or double-length
    UCHAR *pInitKey,      // init key in binary
    UCHAR *pInitKSN       // init key serial number in binary
);

//-----------------------------------------------------------------------------
// IDT_EncryptPIN:
// Encrypt the PIN block which is created by XOR the PIN number and account
// number by using the DUKPT algorithm. After the encryption is performed,
// a new set of future keys are created and the key used for encryption is
// deleted.
// This function take 2 inputs, a PIN and account number.
// Plain text PINs have the following structure:
// 1---5---9---13--17--21--25--29--33--37--41--45--49--53--57--61---
// | C | N | P | P | P | P | PF| PF| PF| PF| PF| PF| PF| PF| F | F |
// -----------------------------------------------------------------
// Where    C => Control field. 0000.
//          N => PIN Length: [0100..1100] (4 to 12)
//          P => PIN Digit [0000..1001] (1 to 9)
//          PF=> PIN Digit or Fill digit, according to length
//          F => Fill Digit. 1111.
// For example, given a PIN of 1234, the input to pPIN should be:
//     0x04, 0x12, 0x34, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF;
// Primary Account Numbers have the following Structure:
// 1---5---9---13--17--21--25--29--33--37--41--45--49--53--57--61---
// | 0 | 0 | 0 | 0 | A1| A2| A3| A4| A5| A6| A7| A8| A9|A10|A11|A12|
// -----------------------------------------------------------------
// Where    0 => Pad Digit. 0000.
//          An=> Primary Account number. The twelve rightmost digits
//               of the primary account number excluding the check digit,
//               which is the last digit.
//
// For example, given an account number of 401234567890, the input to pAcctNum
// should be:
//    0x00, 0x00, 0x40, 0x12, 0x34, 0x56, 0x78, 0x90.
// The current value of KSN is returned in pCurrKSN, while the encrypted PIN
// is returned in pCiphorPin.
//-----------------------------------------------------------------------------
int IDT_EncryptPIN(
    UCHAR *pPin,          // PIN number, 4-16 hex digits
    UCHAR *pAcctNum,      // account number, 10-20 hex digits
    UCHAR *pCurrKSN,            // returns the 10-byte KSN
    UCHAR *pCiphorPin           // returns encrypted PIN, 16 hex digits
    );
*/

class CDUKPT
{
public:
  INT32S  IDT_LoadInitKey(INT8U isTDEA,          // indicate whether the key is single-
                                                 // or double-length
                          INT8U *pInitKey,       // init key in binary
                          INT8U *pInitKSN);      // init key serial number in binary
  INT32S IDT_EncryptPIN(IN  INT8U EncryptType,
						IN  INT32U encrypt_counter,
                        OUT INT8U *pCurrKSN,            // returns the 10-byte KSN
                        OUT INT8U *pCurrKey,          // returns encrypted PIN, 16 hex digits
						IN INT8U PinKey);          // returns encrypted PIN, 16 hex digits
  void IDT_SpecialDEAEncrypt(INT8U *cryptoReg,
                             INT8U *key);
  dukpt_type  dukpt_key;
  INT32U      gkeysize;
  INT32U      gfkeyregsize;
  INT32U      gLRC;
  unsigned int IDT_ChangeCounter(unsigned int encCntr, unsigned char onebit);
  INT32S IDT_GetNumOfOnes(INT32U  encCntr);

private:
  INT8U IDT_CalcLRC (INT8U *key);
  void IDT_GenNewKeys( INT8U   *cryptoReg,
                       INT32U   shiftReg,
                       INT32S   currKeyPtr);
  void IDT_NonReversibleKeyGen(INT8U *cryptoReg,       // crypto register
                               INT8U *key);             // key register
  void IDT_PropogateKeys(INT8U   *cryptoReg,         // crypto register
                         INT32U   shiftReg,           // shift register
                         INT32S   currKeyPtr);         // current key pointer
  void IDT_InitCryptoReg(INT8U   *cyptoReg,
                         INT32U   shiftReg,
                         INT8U   *ksnReg);

  void IDT_UpdateKSN(INT32U   encCntr,
                     INT8U   *pKsnReg);
  //INT8U IDT_CalcLRC(INT8U *key, CPINpad* lpcPINpad);
  INT32U IDT_SetBit(INT32U   reg,
                    INT32S   pos);
  INT32S IDT_GetSetBitPos(INT32U   reg);    // curent value of register
};

//#ifdef __cplusplus
//};
//#endif

#endif  /* _DUKPT_H_ */
