//The abstract class for all kinds of interfaces, such as RS232, USB-HID, USB-HID-KB,
//PS2 KB, USB-CDC.

#if !defined __MSR_INTERFACE__
#define __MSR_INTERFACE__

#define WM_RESCANDEVICE (WM_APP + 1)

#define MAXSENDCOMMANDLENGTH 400

#define ASCII_XON			0x11
#define ASCII_XOFF			0x13
#define ASCII_CR			0x0D

#define STX				0x02
#define ETX				0x03
#define ACK				0x06
#define NAK				0x15
#define KB_ACK			0xFA
#define KB_NAK			0xFD

#define ERROR_COMMAND_SUCCESS 0
#define ERROR_COMMAND_FAILED 1
#define ERROR_COMMAND_EXCEED 2
#define ERROR_COMMAND_WARNING 3

#define IDTECH_USBCDC_PID			0x0530

/* HP support USB HID KB and USB HID */
#define HP_VENDOR_ID			0x03f0 // for HP
#define HP_USBHID_PID			0x2724			
#define HP_MAGTEK_USBHID_PID	0x2724		
#define HP_USBHIDKB_PID			0x1f24 
/* MOIR support USB HID KB and USB HID */
#define MOIR_VENDOR_ID			0x0acd // for MOIR
#define MOIR_USBHID_PID			0x0640			
#define MOIR_MAGTEK_USBHID_PID	0x0640		
#define MOIR_USBHIDKB_PID		0x0620 

#define IDTECH_VENDOR_ID			0x0acd // for Standard
#define IDTECH_USBHID_PID			0x1560 // SecureMag USBHID (ID Tech)
#define	MAGTEK_USBHID_PID			0x2020 // SecureMag USBHID (MagTek)
#define IDTECH_USBHIDKB_PID			0x2030 // SecureMag USBKB
#define IDTECH_USBHIDKB_PID2		0x2120 // ValueMag USBKB
#define IDTECH_USBHID_PID2			0x2110 // ValueMag USBHID
#define MAGTEK_USBHID_PID2			0x2140 // ValueMag USBHID (MagTek)




#define IDTECH_USBHID_PIDSRED   0x2810 // 
 #define IDTECH_USBHIDKB_PIDSRED  0x2820 // 0x2820 //

#define ITF_TYPE_NUM		5

#define HTypeID				0x10
#define BeepID				0x11
#define ChaDelayID			0x12
#define TrackSelectID		0x13

#define DataFmtID			0x15
#define FmtOptionID			0x16
#define TrackSepID			0x17
#define DefaultAllID		0x18
#define SendOptionID		0x19
#define MSRReadingID		0x1A
#define DTEnableSendID		0x1B

#define DecodingMethodID	0x1D
#define SetID				0x1E
#define ReviewID			0x1F

#define TerminatorID		0x21
#define FmVerID				0x22
#define USBHIDFmtID			0x23

#define BaudID				0x41
#define DataID				0x42
#define ParityID			0x43
#define HandID				0x44
#define StopID				0x45

#define XOnID				0x47
#define XOffID				0x48

#define LRCID				0x60
#define T17BStartID			0x61
#define T16BStartID			0x62
#define T15BStartID			0x63
#define T27BStartID			0x64
#define T25BStartID			0x65
#define T37BStartID			0x66
#define T36BStartID			0x67
#define T35BStartID			0x68
#define EndID				0x69

#define BTModeID			0x70	

#define PrefixID			0xD2
#define PostfixID			0xD3
#define AddedFieldID		0xFA
#define SearchCmdID			0xFB
#define SendCmdID			0xFC

///////////////////////////////////////////////////////////////
//New feature
#define PollingIntervalID	0x14
#define ForeignKBID			0x24
#define SecureKeyID			0x25
#define Track1PrefixID		0x34
#define Track2PrefixID		0x35
#define Track3PrefixID		0x36
#define Track1SuffixID		0x37
#define Track2SuffixID		0x38
#define Track3SuffixID		0x39
#define T1EndID				0x69
#define T2EndID				0x6a
#define T3EndID				0x6b

#define		MAX_PREFIX_LENGTH				0x10
#define		MAX_POSTFIX_LENGTH				0x10
#define		MAX_ADDEDFIELD_COUNT			8
#define		MAX_ADDEDFIELD_LENGTH			8
#define		MAX_SENDCMD_COUNT				4
#define		MAX_SENDCMD_LENGTH				40
#define		MAX_SEARCHCMD_COUNT				12
#define		MAX_SEARCHCMD_LENGTH			20
#define		MAX_SENDCMD_FIELD_COUNT			150 
#define		MAX_SENDCMD_FIELD_TOTALCOUNT	160 

#define		FML_BASE			0xE0
#define		FML_CREDITCARD		0xE0
#define		FML_DMV				0xE1
#define		FML_AAMVA			0xE2
#define		FML_CUSTOM			0xE3

#define		LENGTH_MATCH		0xF0
#define		STRING_MATCH		0xF1
#define		SEARCH_BEFORE		0xF2
#define		SEARCH_BETWEEN		0xF3
#define		SEARCH_AFTER		0xF4
#define		SEARCH_AMONG		0xF5

typedef enum
{
	ITF_TYPE_UNKNOWN = 0, 
	ITF_TYPE_RS232 = 1,
	ITF_TYPE_PS2_KB = 2,
	ITF_TYPE_USB_HID_KB = 3,
	ITF_TYPE_USB_VCOM = 4,
	ITF_TYPE_USB_HID = 5
}INTERFACE_TYPE;

static const TCHAR* sc_ItfType_Name[] = {
							_T("Unknown"),
							_T("RS232"),
							_T("PS/2 Keyboard"),
							_T("USB-HID-KB"),
							_T("USB-RS232"),
							_T("USB-HID")
							};

enum emHType {	HT_PCAT = '0', 
				HT_NOTEBOOK = '4',		
				HT_PCAT_WITH_EXTERNAL_KB = '5', 
				HT_PCAT_NO_EXTERNAL_KB = '6'	};	
#define HTYPE_BASE	'0'

enum emBeepVol {	BEEPVOL_OFF = '0', 
					BEEPVOL_LOW = '1', 
					BEEPVOL_HIGH = '2'	};
#define BEEPVOL_BASE	'0'

enum emChaDelay {	CHADELAY_2MS = '0',
					CHADELAY_5MS = '1',
					CHADELAY_10MS = '2',
					CHADELAY_20MS = '3',
					CHADELAY_50MS = '4',
					CHADELAY_100MS = '5'	};
#define CHADELAY_BASE	'0'

enum emHand {	HAND_XON_XOFF = '0', HAND_RTS_CTS = '1'	};
#define HAND_BASE	'0'
static const BYTE sc_byHandShake[] = { 0/*XONXOFF*/, 1/*RTSCTS*/ };

enum emStop {	STOP_1BIT = '0', STOP_2BITS = '1'	};
#define STOP_BASE	'0'
static const BYTE sc_byStopBits[] = { ONESTOPBIT, TWOSTOPBITS};

enum emBaud {	BAUD_2400BPS = '3',
				BAUD_4800BPS = '4',
				BAUD_9600BPS = '5',
				BAUD_19200BPS = '6',
				BAUD_38400BPS = '7'	};
#define BAUD_BASE	'3'
static const DWORD sc_dwBaudRate[] = { 2400, 4800, 9600, 19200, 38400 };

enum emData {	DATA_8BITS = '0', DATA_7BITS = '1'	};
#define DATA_BASE	'0'
static const BYTE sc_byDataBits[] = { 8, 7};

enum emParity {	IDT_PARITY_NONE = '0',
				IDT_PARITY_EVEN = '1',
				IDT_DPARITY_ODD = '2',
				IDT_DPARITY_MARK = '3',
				IDT_DPARITY_SPACE = '4'	};
#define PARITY_BASE	'0'
static const BYTE sc_byParity[] = { NOPARITY, EVENPARITY, ODDPARITY, MARKPARITY, SPACEPARITY };

enum emTrackSelect {	TRACKSELECT_ANY_TRACK = '0',
						TRACKSELECT_TRACK1_ONLY	= '1',
						TRACKSELECT_TRACK2_ONLY = '2',
						TRACKSELECT_TRACK1_2 = '3',
						TRACKSELECT_TRACK3_ONLY = '4',
						TRACKSELECT_TRACK1_3 = '5',
						TRACKSELECT_TRACK2_3 = '6',
						TRACKSELECT_ALL3TRACKS = '7',
						TRACKSELECT_TRACK1OR2 = '8',
						TRACKSELECT_TRACK2OR3 = '9'
					};
#define TRACKSELECT_BASE	'0'

enum emDataFmt {	DATAFMT_IDTECH = '0',
					DATAFMT_UIC = '1',
					DATAFMT_MAGTEK = '2'	};
#define DATAFMT_BASE	'0'

typedef struct _FMT_OPTION
{
	BYTE Not_Send_LRC_UIC		: 1;	//bit0
	BYTE Buffer_Mode_MagTek		: 1;	//bit1
	BYTE Command_Mode_MagTek	: 1;	//bit2: 1 = +/-, 0 = I/R
	BYTE Not_Send_STX_MagTek	: 1;	//bit3
	BYTE Not_Send_ESC_MagTek	: 1;	//bit4
	BYTE Not_Send_CR_MagTek		: 1;	//bit5
	BYTE Not_Send_ETX_MagTek	: 1;	//bit6
	BYTE Padding				: 1;	
}FMT_OPTION, *PFMT_OPTION;

static const int	TERMINATOR_CRLF = 0xd;
static const int	TERMINATOR_CR = 0xd;
static const int	TERMINATOR_LF = 0xa;
static const int	TERMINATOR_NONE = 0;

enum emDTEnableSend {	DATAEDIT_DISABLE = '0',
						DATAEDIT_ENABLE_MATCH = '1',
						DATAEDIT_ENABLE_UNMATCH = '3'	};

#define DTENABLESEND_BASE	'0'

enum emSendOption {	SENDOPTION_NOSENDSE_ALLDATA = '0',
					SENDOPTION_SENDSE_ALLDATA = '1',
					SENDOPTION_NOSENDSE_ACCOUNTNO = '2',
					SENDOPTION_SENDSE_ACCOUNTNO = '3'	};

#define SENDOPTION_BASE	'0'

enum emMSRReading {	MSRREADING_DISABLE = '0',
					MSRREADING_ENABLE = '1'	};
#define MSRREADING_BASE	'0'

enum emDecodingMethod {	DM_DECODING_RAW_DATA = '0',
						DM_DECODING_BOTH_DIRECT = '1',
						DM_MOVING_ALONG_HEAD = '2',
						DM_MOVING_ALONG_HEAD_AGAINST = '3'	};
#define DECODINGMETHOD_BASE	'0'

enum emLRC { LRC_NO = '0', LRC_YES = '1'	};
#define LRC_BASE	'0'

#define USBHIDFMT_IDTECH  '0' //ID TECH Format
#define	USBHIDFMT_MAGTEK  '1' // Meg Tech formst.
#define	USbHIDFMT_USBKB   '8' // usb keyboard format.
static int Hid_Fmts[] = {
					0, USBHIDFMT_IDTECH, 
					1, USBHIDFMT_MAGTEK,
					2, USbHIDFMT_USBKB, 
					-1,-1
	};
int getHidFmt(int index);
int getIndexFromFmt(int fmt);


static const LPCTSTR scTL_HType[] = {
						_T("IBM PC/AT"),
						_T(""),
						_T(""),
						_T(""),
						_T(""),
						_T("IBM PC/AT, External Keyboard"),
						_T("IBM PC/AT, NO External Keyboard")
						};
static const BYTE scVL_HType[] = {
						HT_PCAT,
						HT_PCAT_WITH_EXTERNAL_KB,
						HT_PCAT_NO_EXTERNAL_KB
						};

static const LPCTSTR scTL_BeepVol[] = {
						_T("OFF"),
						_T("LOW"),
						_T("HIGH")
						};
static const BYTE scVL_BeepVol[] = {
						BEEPVOL_OFF,
						BEEPVOL_LOW,
						BEEPVOL_HIGH
						};

static const LPCTSTR scTL_ChaDelay[] = {
						_T("2 ms"),
						_T("5 ms"),
						_T("10 ms"),
						_T("20 ms"),
						_T("50 ms"),
						_T("100 ms")
						};
static const BYTE scVL_ChaDelay[] = {
						CHADELAY_2MS,
						CHADELAY_5MS,
						CHADELAY_10MS,
						CHADELAY_20MS,
						CHADELAY_50MS,
						CHADELAY_100MS
						};

static const LPCTSTR scTL_Hand[] = {
						_T("X-On/X-Off"),
						_T("RTS/CTS")
						};
static const BYTE scVL_Hand[] = {
						HAND_XON_XOFF,
						HAND_RTS_CTS
						};

static const LPCTSTR scTL_Stop[] = {
						_T("1 Bit"),
						_T("2 Bits")
						};
static const BYTE scVL_Stop[] = {
						STOP_1BIT,
						STOP_2BITS
						};

static const LPCTSTR scTL_Baud[] = {
						_T("2400"),
						_T("4800"),
						_T("9600"),
						_T("19200"),
						_T("38400")
						};
static const BYTE scVL_Baud[] = {
						BAUD_2400BPS,
						BAUD_4800BPS,
						BAUD_9600BPS,
						BAUD_19200BPS,
						BAUD_38400BPS
						};

static const LPCTSTR scTL_Data[] = {
						_T("8 Bits"),
						_T("7 Bits")
						};
static const BYTE scVL_Data[] = {
						DATA_8BITS,
						DATA_7BITS
						};

static const LPCTSTR scTL_Parity[] = {
						_T("None"),
						_T("Even"),
						_T("Odd"),
						_T("Mark"),
						_T("Space")
						};
static const BYTE scVL_Parity[] = {
						IDT_PARITY_NONE,
						IDT_PARITY_EVEN,
						IDT_DPARITY_ODD,
						IDT_DPARITY_MARK,
						IDT_DPARITY_SPACE
						};

static const LPCTSTR scTL_TrackSelect[] = {
						_T("Any Track"),
						_T("Track1 Only"),
						_T("Track2 Only"),
						_T("Track1 And Track2"),
						_T("Track3 Only"),
						_T("Track1 And Track3"),
						_T("Track2 And Track3"),
						_T("All Three Tracks"),
						_T("Track1 Or Track2"),
						_T("Track2 Or Track3")
						};
static const BYTE scVL_TrackSelect[] = {
						TRACKSELECT_ANY_TRACK,
						TRACKSELECT_TRACK1_ONLY,
						TRACKSELECT_TRACK2_ONLY,
						TRACKSELECT_TRACK3_ONLY,
						TRACKSELECT_TRACK1_2,
						TRACKSELECT_TRACK1_3,
						TRACKSELECT_TRACK2_3,
						TRACKSELECT_ALL3TRACKS,
						TRACKSELECT_TRACK1OR2,
						TRACKSELECT_TRACK2OR3
						};

static const LPCTSTR scTL_DataFmt[] = {
						_T("Standard Format"),
						_T("Default Format"),
						_T("Default-2 Format")
						};
static const BYTE scVL_DataFmt[] = {
						DATAFMT_IDTECH,
						DATAFMT_UIC,
						DATAFMT_MAGTEK
						};

static const LPCTSTR scTL_MSRReading[] = {
						_T("Disabled"),
						_T("Enabled")
						};
static const BYTE scVL_MSRReading[] = {
						MSRREADING_DISABLE,
						MSRREADING_ENABLE
						};


static const LPCTSTR scTL_DecodingMethod[] = {
						_T("Raw Data, Not Decoded"),
						_T("Decoding In Two Swiping Direction"),
						_T("Magnetic Head Move Along Encoding Way"),
						_T("Magnetic Head Move Against Encoding Way")
						};
static const BYTE scVL_DecodingMethod[] = {
						DM_DECODING_RAW_DATA,
						DM_DECODING_BOTH_DIRECT,
						DM_MOVING_ALONG_HEAD,
						DM_MOVING_ALONG_HEAD_AGAINST
						};

static const LPCTSTR scTL_LRC[] = {
						_T("Not Send LRC"),
						_T("Send LRC")
						};
static const BYTE scVL_LRC[] = {
						LRC_NO,
						LRC_YES
						};

static const LPCTSTR scTL_USBHIDFmt[] = {
						_T("Standard USB HID Data Format"),
						_T("Default USB HID Data Format")
						};
static const BYTE scVL_USBHIDFmt[] = {
						USBHIDFMT_IDTECH,
						USBHIDFMT_MAGTEK,						
						};

static const LPCTSTR scTL_LRC_UIC[] = {
						_T("Send  LRC"),
						_T("Do Not Send LRC")
						};
static const BYTE scVL_LRC_UIC[] = { 0, 1};


static const LPCTSTR scTL_Buffer_MagTek[] = {
						_T("Unbuffered mode"),
						_T("Buffered mode")
						};
static const BYTE scVL_Buffer_MagTek[] = { 0, 1};


static const LPCTSTR scTL_Command_MagTek[] = {
						_T("I/R command"),
						_T("+/- command")
						};
static const BYTE scVL_Command_MagTek[] = { 0, 1 };


static const LPCTSTR scTL_STX_MagTek[] = {
						_T("Send STX"),
						_T("Not send STX")
						};
static const BYTE scVL_STX_MagTek[] = { 0, 1 };


static const LPCTSTR scTL_ESC_MagTek[] = {
						_T("Send  ESC"),
						_T("Not send ESC")
						};
static const BYTE scVL_ESC_MagTek[] = { 0, 1 };


static const LPCTSTR scTL_CR_MagTek[] = {
						_T("Send CR"),
						_T("Not send CR")
						};
static const BYTE scVL_CR_MagTek[] = { 0, 1 };


static const LPCTSTR scTL_ETX_MagTek[] = {
						_T("Send ETX"),
						_T("Not send ETX")
						};
static const BYTE scVL_ETX_MagTek[] = { 0, 1 };


static const LPCTSTR scTL_DEEnable[] = {
						_T("Disable"),
						_T("Enable")
						};
static const BYTE scVL_DEEnable[] = { 0, 1 };


static const LPCTSTR scTL_DEUnmatched[] = {
						_T("Matched"),
						_T("Unmatched")
						};
static const BYTE scVL_DEUnmatched[] = { 0, 1 };


static const LPCTSTR scTL_SOSendSE[] = {
						_T("NOT Send Start/End Sentinel"),
						_T("Send Start/End Sentinel")
						};
static const BYTE scVL_SOSendSE[] = { 0, 1 };

static const LPCTSTR scTL_SOSendT2A[] = {
						_T("Send All Data For Track2"),
						_T("Only Send Account Number For Track2")
						};
static const BYTE scVL_SOSendT2A[] = { 0, 1 };

static const LPCTSTR scTL_SOSendErrorIndication[] = {
						_T("Not Send Error Indication"),
						_T("Send Error Indication")
						};
static const BYTE scVL_SOSendErrorIndication[] = { 0, 1 };

typedef struct _FID_ITEM
{
	BYTE	byFuncID;
	BYTE	byType; //Combo=0, Edit-Text=1, Edit-Num=2, Edit-Other=3, BitSet=0xF
	WORD	wValidBits; //If not 0, it has sub FID_ITEM struct.
	LPTSTR	pszFuncName;
	ULONG	ulMaxLen; //The max length of the value, in bytes

	PVOID	DefData;
	BYTE	byBaseData; // For BitSet! it will be added to each FuncData bytes! 

	const LPCTSTR*	TextList;
	const BYTE*		ValueList;
	int				nElemCnt;

} FID_ITEM, *PFID_ITEM;

enum HANDSHAKE { XONXOFF=0, RTSCTS=1};
typedef struct _RS232_PARAMS
{
	DWORD dwBaudRate;
	BYTE byDataBits;
	BYTE byStopBits;
	BYTE byParity;
	BYTE byHandShake; // 0: X-On/X-Off; 1: RTS/CTS
	BYTE byXonChar;
	BYTE byXoffChar;
}RS232_PARAMS, *PRS232_PARAMS;

//============================================================================================

typedef struct _MSR_DEV_ATTR
{
	USHORT	VendorID;
	USHORT	ProductID;
	USHORT	VersionNumber;
	//...
}MSR_DEV_ATTR, *PMSR_DEV_ATTR;

typedef struct _MSR_NAME
{
	INTERFACE_TYPE emItfType;
	CString strDevName;
	ULONG	ulIndex;
	CString strFmVer; 
}MSR_NAME, *PMSR_NAME;

//============================================================================================

class CMSRItf
{
public:
	CMSRItf() {m_FirmwareVersion = "";}
	virtual ~CMSRItf() {}
//	virtual BOOL GetFirmware(CString &strFirmware) = 0;
	virtual BOOL OpenMSRDevice( ULONG ulIndex ) = 0;
	virtual BOOL OpenMSRDeviceByName( LPCTSTR lpszName) = 0;
	virtual BOOL CloseMSRDevice() = 0;
	virtual BOOL SendCommand(BYTE* pCmdBuf,ULONG cbCmdBuf,PBYTE pRetBuf,PULONG pcbRetBuf) = 0;

	virtual INTERFACE_TYPE GetInterfaceType() = 0;
	virtual HANDLE GetMSRHandle() = 0;
	virtual void GetMSRName(CString& strDevName) = 0;
	CString m_FirmwareVersion;
};


//============================================================================================
CMSRItf* NEW_MSRITF(INTERFACE_TYPE emItfType);
VOID DEL_MSRITF(CMSRItf* pMSRItf);

#endif //__MSR_INTERFACE__